import { useMemo } from "react";

import { useWatch } from "react-hook-form";

import { cn } from "@/lib/utils";

import { Field } from "./types/all-fields";

const FormBuilder = <T extends Record<string, Field<T>>>({
	fields,
	FormHooks,
	containerClassName,
	className,
}: FormBuilderProps<T>) => {
	return (
		<div
			className={cn(
				"grid w-full grid-cols-12 gap-3",
				containerClassName,
				className
			)}
		>
			{Object.entries(fields)?.map(([key, field]) => (
				<WatchHelper
					key={key}
					FormHooks={FormHooks}
					fieldsToWatch={field.fieldsToWatch ? field.fieldsToWatch : []}
					field={field}
					FieldName={key}
				/>
			))}
		</div>
	);
};

function WatchHelper<T extends Record<string, Field<T>>>({
	FormHooks,
	fieldsToWatch = [],
	field,
	FieldName,
}: {
	FormHooks: UseFormReturn<T>;
	fieldsToWatch: string[];
	field: Field<T>;
	FieldName: string;
}) {
	const WatchValues = useWatch({
		control: FormHooks.control,
		disabled: !fieldsToWatch.length,
		name: fieldsToWatch,
	});
	const WatchValuesObject = useMemo(() => {
		const _ob: any = {};
		fieldsToWatch.forEach((field, index) => {
			_ob[field] = WatchValues[index];
		});
		return _ob;
	}, [fieldsToWatch, WatchValues]);
	return (
		<field.component
			name={FieldName}
			control={FormHooks.control}
			{...(typeof field.componentProps === "function"
				? field.componentProps({
					...FormHooks,
					WatchValues: WatchValuesObject,
				})
				: field.componentProps)}
		/>
	);
}

export default FormBuilder;
